/***************************************************************/
/* Dustin Graves                                               */
/* CSCI 185 10                                                 */
/* 9/30/98                                                     */
/***************************************************************/

import java.awt.*;

//class for geometric transformations
public class Xform
{
  //Instance variables
  protected double fMatrix[][];
  
  //Constructors
  public Xform ()
  {
    fMatrix = new double[3][3];
    setIdentity ();
  }

  //Instance methods
  
  //Set the Identity Matrix
  public void setIdentity ()
  {
    int i, j;

    for (i = 0; i < 3; i++)
      for (j = 0; j < 3; j++)
        fMatrix[i][j] = (i == j) ? 1.0 : 0.0;
  }

  //Set ths Xform to Translate
  public void setTranslate (double tx, double ty) 
  {
    setIdentity ();
    fMatrix[0][2] = tx;
    fMatrix[1][2] = ty;
  }

  //Set ths Xform to Scale
  public void setScale (double sx, double sy) 
  {
    setIdentity ();
    fMatrix[0][0] = sx;
    fMatrix[1][1] = sy;
  }

  //Set ths Xform to Rotate
  public void setRotate (double theta) 
  {
    //Angle in radians
    double ro = theta * (Math.PI / 180);

    setIdentity ();
    fMatrix[0][0] = Math.cos (ro);
    fMatrix[0][1] = -Math.sin (ro);
    fMatrix[1][0] = Math.sin (ro);
    fMatrix[1][1] = Math.cos (ro);
  }

  //If this is A and in is B operation is B*A
  public void preCompose (Xform in)
  {
     double temp[][] = new double[3][3];
     int r,
         c;

     for (r = 0; r < 3; r++)
     {
        for (c = 0; c < 3; c++)
        {
          temp[r][c] = in.fMatrix[r][0] * fMatrix[0][c] + 
                       in.fMatrix[r][1] * fMatrix[1][c] +
                       in.fMatrix[r][2] * fMatrix[2][c];
        }
     }

     for (r = 0; r < 3; r++)
       for (c = 0; c < 3; c++)
         fMatrix[r][c] = temp[r][c];
  }

  //If this is A and in is B operation is A*B
  public void postCompose (Xform in)
  {
     double temp[][] = new double[3][3];
     int r,
         c;

     for (r = 0; r < 3; r++)
     {
        for (c = 0; c < 3; c++)
        {
          temp[r][c] = fMatrix[r][0] * in.fMatrix[0][c] + 
                       fMatrix[r][1] * in.fMatrix[1][c] +
                       fMatrix[r][2] * in.fMatrix[2][c];
        }
     }

     for (r = 0; r < 3; r++)
       for (c = 0; c < 3; c++)
         fMatrix[r][c] = temp[r][c];
  }

  //Apply this Xform to a point
  public Point2d apply (Point2d in)
  {
    double temp;

    temp = (fMatrix[0][0] * in.fX) +
           (fMatrix[0][1] * in.fY) +
           (fMatrix[0][2] * in.fH);

    in.fY = (fMatrix[1][0] * in.fX) +
            (fMatrix[1][1] * in.fY) +
            (fMatrix[1][2] * in.fH);

    in.fX = temp;

    return in;
  }

} //Xform



